/* 
 * This file is part of "Modulos de entrenamiento para FPGAs"
 * Copyright (c) 2018 Miguel Angel Rodriguez Jodar.
 * 
 * This program is free software: you can redistribute it and/or modify  
 * it under the terms of the GNU General Public License as published by  
 * the Free Software Foundation, version 3.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

/*
   EJEMPLO 8
   ---------
   
   Este código contiene un ejemplo de uso de los módulos de entrenamiento para FPGA, y
   está escrito para ser incluido dentro de un módulo (ver tld_modulos_entrenamiento_*.v)
   No es por tanto un código Verilog completo.
   
   Dentro de este fichero se pueden definir wire y reg, así como instanciar al módulo
   "display" que es el módulo principal del conjunto de módulos de entrenamiento.
   
   Dentro de este fichero no se pueden definir otros módulos o entidades, ya que este 
   fichero en sí está dentro de un módulo preexistente.

   Mejoramos un poco el cronómetro del EJEMPLO 7. Lo que vamos a hacer es conseguir que
   el cronómetro cuente en decimal y no en hexadecimal. Para ello usaremos 4 digitos
   (los 4 centrales del display) para almacenar unidades, decenas, centenas y unidades
   de millar, y un quinto dígito a la derecha de los anteriores, para guardar décimas de
   segundo, como en el ejemplo anterior. El procedimiento es análogo al que hemos usado para
   contar décimas de segundo, que al llegar a 9 pasan a 0 y hacen incrementar los segundos.
   En este caso haremos incrementos en cadena: cuando las décimas valen 9, hacen incrementar
   a las unidades. Cuando las unidades valen 9, hacen incrementar las decenas. Cuando las
   decenas valen 9, hacen incrementar las centenas, etc.
   De esta manera, el cronómetro puede contar hasta 9999.9 segundos.
*/   


module ejemplo08 
(
input wire clk,
output wire [7:0]leds, //= 8'b10000000

	output wire [6:0] seg_0,
	output wire [6:0] seg_1,
	output wire [6:0] seg_2,
	output wire [6:0] seg_3,
	output wire [6:0] seg_4,
	output wire [6:0] seg_5,
	output wire	[6:0] seg_6,
	output wire [6:0] seg_7,
	output reg [7:0] punto
);
  reg [7:0]leds_reg = 8'b10000000; //creamos los leds_reg y encendemos en bit más significativo
  assign leds = leds_reg; //hacemos una asignación al wire externo
  
  reloj25mhz r25mhz (.inclk0(clk),.c0(clk25m)); //ajustamos el reloj a 25Mhz 
 
  reg sentido = 1'b0;  // en este flipflop guardamos el sentido: 0=derecha, 1=izquierda
  reg [23:0] contador = 24'd0;  // contador que cuenta de 0 hasta 2500000 (una décima de segundo, a 25 MHz)

  // Valores iniciales del cronómetro
  reg [3:0] decimas  = 4'd0;
  reg [3:0] unidades = 4'd0;
  reg [3:0] decenas  = 4'd0;
  reg [3:0] centenas = 4'd0;
  reg [3:0] millares = 4'd0;
  
  always @(posedge clk25m) begin   // en cada ciclo de reloj del reloj de 25 MHz hacemos lo siguiente:
    if (contador == 24'd2500000) begin   // si hemos llegado a una décima de segundo de tiempo...
      contador <= 24'd0;                 //    ponemos el contador a 0, y

      // Gestión del cronómetro
      if (decimas == 4'd9) begin         // Vamos a pasar de 9 a 10 en las decimas? Si es que si,
        decimas <= 4'd0;                 //    ponemos las décimas a 0
        if (unidades == 4'd9) begin      //    Vamos a pasar de 9 a 10 en las unidades? Si es que sí,
          unidades <= 4'd0;              //        ponemos las unidades a 0
          if (decenas == 4'd9) begin     //        Vamos a pasar de 9 a 10 en las decenas? .... etc .....
            decenas <= 4'd0;
            if (centenas == 4'd9) begin
              centenas <= 4'd0;
              if (millares == 4'd9) begin
                millares <= 4'd0;
              end
              else
                millares <= millares + 4'd1;
            end
            else
              centenas <= centenas + 4'd1;
          end
          else
            decenas <= decenas + 4'd1;   //        Si es que no, incrementamos las decenas
        end
        else
          unidades <= unidades + 4'd1;   //    Si es que no, incrementamos las unidades
      end
      else
        decimas <= decimas + 4'd1;       // Si es que no, incrementamos las decimas
                
      // Gestión de la tira de leds
      if (sentido == 1'b0)               //    miramos en qué sentido hemos de rotar
        leds_reg <= {leds_reg[0], leds_reg[7:1]};    //      rotamos a la derecha si es 0
      else                               //      si no,
        leds_reg <= {leds_reg[6:0], leds_reg[7]};    //      entonces rotamos a la izquierda
      if (leds_reg[1] == 1'b1 && sentido == 1'b0)       // si estamos a punto de llegar a la esquina derecha y estábamos rotando a la derecha...
        sentido <= 1'b1;                            //    pasamos a rotar a la izquierda
      else if (leds_reg[6] == 1'b1 && sentido == 1'b1)  // si no, si estamos a punto de llegar a la esquina izquierda y estábamos rotando a la izquierda...
        sentido <= 1'b0;                            //    pasamos a rotar a la derecha
    end
    else                                 // en otro caso
      contador <= contador + 24'd1;      //    incrementamos el contador
		
		 punto = ~8'b00010000; //apagamos todos los puntos de los BCD menos el de las dédimas
  end
  
llamardisplay display_00 (.binario(millares), .display(seg_0));
llamardisplay display_01 (.binario(centenas), .display(seg_1));
llamardisplay display_02 (.binario(decenas), .display(seg_2));
llamardisplay display_03 (.binario(unidades), .display(seg_3));
llamardisplay display_04 (.binario(decimas), .display(seg_4));
llamardisplay display_05 (.binario(0), .display(seg_5));
llamardisplay display_06 (.binario(0), .display(seg_6));
llamardisplay display_07 (.binario(0), .display(seg_7));

endmodule
  